"""
Test x12/base
"""
import datetime
from decimal import Decimal
from typing import Annotated, get_type_hints, TypeAlias

from pytest import fixture, CaptureFixture, mark, raises

from x12 import base
from x12.annotations import *

def test_source_typical():
    s = base.Source("\r\rISA|00|01|02|:~   \r  SEG|03|04|05~\r")
    assert s.peek(3) == "ISA"
    s.element_sep = "|"
    s.array_sep = ":"
    s.segment_sep = "~"
    assert s.next_segment() == "ISA"
    assert s.elements() == ["ISA", "00", "01", "02", ":"]
    assert s.peek(3) == "SEG"
    assert s.next_segment() == "SEG"
    assert s.elements() == ["SEG", "03", "04", "05"]


def test_source_edge():
    """A typical use without peeking."""
    s = base.Source("\n\n\nISA|00|01|02|:~   \n  SEG|03|04|05~\n\n", element_sep = "|", segment_sep = "~")
    assert s.elements() == ["ISA", "00", "01", "02", ":"]
    assert repr(s) == "self.element_sep='|' self.segment_sep='~' text='\\n\\n\\nISA|00|01|02|:~' && '   \\n  SEG|03|04|05~\\n\\n'"


def test_schema():
    class SomeSegment(base.Segment):
        """
        SomeSegment
        """
        a: Annotated[str, MinLen(1)]
        BItem: TypeAlias = Annotated[str, MinLen(2)]
        b: list[BItem, MaxItems(2)]
        c: Annotated[int, Title("c")] | None
        d: Annotated[float, Title("d")]
        e: Annotated[datetime.date, Title("e")]
        f: Annotated[datetime.time, Title("f")]
        g: Annotated[Decimal, Title("g"), Scale(2)]
    assert base.schema(SomeSegment) == {
        'class_name': 'SomeSegment',
        'description': 'SomeSegment',
        'properties': {
                    'a': {'minLength': 1, 'type': 'string'},
                    'b': {'items': {'minLength': 2, 'type': 'string'},
                          'maxItems': 2,
                          'type': 'array'},
                    'c': {'any': [{'title': 'c', 'type': 'integer'},
                                  {'type': 'null'}]},
                    'd': {'title': 'd', 'type': 'number'},
                    'e': {'convert': 'datetime.date',
                          'pyformat': '%Y%m%d',
                          'title': 'e',
                          'type': 'string'},
                    'f': {'convert': 'datetime.time',
                          'pyformat': '%h%m',
                          'title': 'f',
                          'type': 'string'},
                    'g': {'convert': 'Decimal',
                          'scale': '?',
                          'title': 'g',
                          'type': 'string',
                          'x-scale': 2}
        },
        'type': 'object'
    }


def test_composite_padding_annotations():
    class SomeComposite(base.Composite):
        se: Annotated[str, Title("Some Element"), MinLen(2), MaxLen(2)]
    class SomeSegment(base.Segment):
        c: SomeComposite
    class SomeLoop(base.Loop):
        s: SomeSegment
    class SomeMessage(base.Message):
        l: SomeLoop

    p = base.X12Parser(SomeMessage)
    c = p.composite_build(SomeComposite, ["01"])
    assert repr(c) == "SomeComposite(se='01')"
    assert c.se == "01"
    assert c.elements() == ['01']
    assert base.asdict(c) == {'_kind': 'Composite', '_name': 'SomeComposite', 'se': '01'}

    assert base.schema(SomeComposite) == {
        'class_name': 'SomeComposite',
        'description': None,
        'properties': {
            'se': {
                'maxLength': 2,
                'minLength': 2,
                'title': 'Some Element',
                'type': 'string'}
            },
        'type': 'object'
    }


def test_composite_build_2():
    class SomeComposite(base.Composite):
        se: Annotated[str, Title("Some Element"), MinLen(2), MaxLen(2)] | None
        be: int
    class SomeSegment(base.Segment):
        c: SomeComposite
    class SomeLoop(base.Loop):
        s: SomeSegment
    class SomeMessage(base.Message):
        l: SomeLoop

    p = base.X12Parser(SomeMessage)
    c = p.composite_build(SomeComposite, ["01", "42"])

    assert repr(c) == "SomeComposite(se='01', be=42)"
    assert c.se == "01"
    assert c.be == 42
    assert c.elements() == ['01', 42]
    assert base.asdict(c) == {'_kind': 'Composite', '_name': 'SomeComposite', 'se': '01', 'be': 42}

    assert base.schema(SomeComposite) == {
        'class_name': 'SomeComposite',
        'description': None,
        'properties': {
            'se': {
                'any': [
                    {'maxLength': 2,
                     'minLength': 2,
                     'title': 'Some Element',
                     'type': 'string'},
                    {'type': 'null'}
                ]
            },
            'be': {'type': 'integer'},
        },
        'type': 'object'
    }


def test_composite_validation():
    class SomeComposite(base.Composite):
        se: Annotated[str, Title("Some Element"), MinLen(2), MaxLen(2), Enumerated("WX", "YZ")]
    class SomeSegment(base.Segment):
        c: SomeComposite
    class SomeLoop(base.Loop):
        s: SomeSegment
    class SomeMessage(base.Message):
        l: SomeLoop

    p = base.X12Parser(SomeMessage)
    with raises(ValueError) as info:
        c = p.composite_build(SomeComposite, ["AB"])
    assert info.value.args == ("invalid SomeComposite.se: typing.Annotated[str, Title('Some Element'), "
         'MinLen(2), MaxLen(2), Enumerated(\'WX\', \'YZ\')] ("invalid value for '
         '\'AB\', not in (\'WX\', \'YZ\')", \'Enumerated\')',)

def test_composite_skipvalidation_ok():
    class SomeComposite(base.Composite):
        se: Annotated[str, Title("Some Element"), MinLen(2), MaxLen(2), Enumerated("WX", "YZ")]
    class SomeSegment(base.Segment):
        c: SomeComposite
    class SomeLoop(base.Loop):
        s: SomeSegment
    class SomeMessage(base.Message):
        l: SomeLoop

    p = base.X12Parser(SomeMessage, skip_validation=["SomeSegment:se:Enumerated"])
    c = p.composite_build(SomeComposite, ["AB"])
    assert c.se == "AB"

def test_composite_skipvalidation_notok():
    class SomeComposite(base.Composite):
        se: Annotated[str, Title("Some Element"), MinLen(2), MaxLen(2), Enumerated("WX", "YZ")]
    class SomeSegment(base.Segment):
        c: SomeComposite
    class SomeLoop(base.Loop):
        s: SomeSegment
    class SomeMessage(base.Message):
        l: SomeLoop

    p = base.X12Parser(SomeMessage, skip_validation=["SomeSegment:se:MinLen"])
    with raises(ValueError) as info:
        c = p.composite_build(SomeComposite, ["AB"])
    assert info.value.args == ("invalid SomeComposite.se: typing.Annotated[str, Title('Some Element'), "
         'MinLen(2), MaxLen(2), Enumerated(\'WX\', \'YZ\')] ("invalid value for '
         '\'AB\', not in (\'WX\', \'YZ\')", \'Enumerated\')',)

def test_segment_annotated():
    # Given
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
        e02: Annotated[str, Title("I02"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="AN")]
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01|02|:~", element_sep="|", array_sep=":", segment_sep="~")
    # When
    # s = SomeSegment.parse(src)
    p = base.X12Parser(SomeMsg)
    s = p.segment_parse(SomeSegment, src)
    # Then
    assert s.e01 == "01"
    assert s.e02 == "02"
    assert s.elements() == ['01', '02']
    assert repr(s) == "SomeSegment(e01='01', e02='02')"
    assert base.asdict(s) == {'_kind': 'Segment', '_name': 'SomeSegment', 'e01': '01', 'e02': '02'}

    assert base.schema(SomeSegment) == {
        'class_name': 'SomeSegment',
        'description': 'Some Segment',
        'type': 'object',
        'properties': {
            'e01': {'title': 'I01', 'type': 'string', 'data_type_code': 'ID', 'minLength': 2, 'maxLength': 2},
            'e02': {'title': 'I02', 'type': 'string', 'data_type_code': 'AN', 'minLength': 2, 'maxLength': 2}
        }
    }

def test_segment_validation():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
        e02: Annotated[str, Title("I02"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="AN")]
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01tooLong|x|:~", element_sep="|", array_sep=":", segment_sep="~")

    # SomeSegment.configure([])
    p = base.X12Parser(SomeMsg)
    with raises(ValueError) as info:
        # s = SomeSegment.parse(src)
        s = p.segment_parse(SomeSegment, src)
    assert info.value.args == ("invalid SomeSegment.e01: typing.Annotated[str, Title('I01'), MinLen(2), "
         'MaxLen(2), OtherMeta(data_type_code=\'ID\')] ("invalid length for '
         '\'01tooLong\', greater than than 2", \'MaxLen\')',)


def test_segment_skipvalidation_ok():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
        e02: Annotated[str, Title("I02"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="AN")]
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01tooLong|x|:~", element_sep="|", array_sep=":", segment_sep="~")
    skip_validation = ["SomeSegment:e01:MaxLen", "SomeSegment:e02:MinLen"]

    # SomeSegment.configure(skip_validation)
    p = base.X12Parser(SomeMsg, skip_validation)
    # s = SomeSegment.parse(src)
    s = p.segment_parse(SomeSegment, src)

    assert s.e01 == "01tooLong"
    assert s.e02 == "x"

def test_segment_skipvalidation_notok():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
        e02: Annotated[str, Title("I02"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="AN")]
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01tooLong|x|:~", element_sep="|", array_sep=":", segment_sep="~")
    skip_validation = ["SomeSegment:e01:Enumerated", "SomeSegment:e02:Enumerated"]

    # SomeSegment.configure(skip_validation)
    p = base.X12Parser(SomeMsg, skip_validation)
    with raises(ValueError) as info:
        # s = SomeSegment.parse(src)
        s = p.segment_parse(SomeSegment, src)
    assert info.value.args == ("invalid SomeSegment.e01: typing.Annotated[str, Title('I01'), MinLen(2), "
         'MaxLen(2), OtherMeta(data_type_code=\'ID\')] ("invalid length for '
         '\'01tooLong\', greater than than 2", \'MaxLen\')',)


def test_segment_make_helpers_annotated():
    class SomeComposite(base.Composite):
        c01: Annotated[int, Title("C01")]
        c02: Annotated[int, Title("C02")]

    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None
        e08: Annotated[datetime.date, Title("I08"), MinLen(6), MaxLen(6), OtherMeta(data_type_code="DT")]
        e16: Annotated[Decimal, Scale(0), Title("I16"), MinLen(1), MaxLen(6), OtherMeta(data_type_code="N0")]
        SomeItem: TypeAlias = Annotated[str, Title("Repeating")]
        e99: Annotated[list[SomeItem], MaxItems(2)]
        e98: int
        e97: SomeComposite
        e96: list[int]
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop

    # SomeSegment.configure([])
    p = base.X12Parser(SomeMsg)

    assert list(SomeSegment._helpers.keys()) == ['e01', 'e08', 'e16', 'e99', 'e98', 'e97', 'e96']

def test_segment_attr_build_union():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01~", element_sep="|", array_sep=":", segment_sep="~")

    # SomeSegment.configure([])
    p = base.X12Parser(SomeMsg)

    # s = SomeSegment.parse(src)
    s = p.segment_parse(SomeSegment, src)
    assert s.e01 == "01"


def test_segment_attr_build_union_empty():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop

    # SomeSegment.configure([])
    p = base.X12Parser(SomeMsg)

    # values = [
    #     SomeSegment.attr_build(field_type, [])
    #     for name, field_type in base.class_fields(SomeSegment)
    # ]
    values = [
        p.segment_attr_build(SomeSegment, name, field_type, [])
        for name, field_type in base.class_fields(SomeSegment)
    ]
    assert values == [None]

def test_segment_attr_build_composite():
    class SomeComposite(base.Composite):
        c01: Annotated[str, Title("C01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None

    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        comp: SomeComposite
        e02: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None
    class SomeLoop(base.Loop):
        s01: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01|02~", element_sep="|", array_sep=":", segment_sep="~")

    # SomeSegment.configure([])
    p = base.X12Parser(SomeMsg)

    # s = SomeSegment.parse(src)
    s = p.segment_parse(SomeSegment, src)
    assert s.e02 == "02"
    assert s.comp.c01 == "01"


def test_segment_schema_annotated():
    class SomeSegment(base.Segment):
        """
        Some Segment
        """
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")] | None
        e08: Annotated[datetime.date, Title("I08"), MinLen(6), MaxLen(6), OtherMeta(data_type_code="DT")]
        e16: Annotated[Decimal, Scale(0), Title("I16"), MinLen(1), MaxLen(6), OtherMeta(data_type_code="N0")]

    assert base.schema(SomeSegment) == {
        'class_name': 'SomeSegment',
        'description': 'Some Segment',
        'type': 'object',
        'properties': {
            'e01': {
                'any': [
                    {
                        'data_type_code': 'ID',
                        'maxLength': 2,
                         'minLength': 2,
                         'title': 'I01',
                         'type': 'string'
                    },
                    {'type': 'null'}
                ]
            },
            'e08': {'convert': 'datetime.date',
                    'data_type_code': 'DT',
                    'maxLength': 6,
                    'minLength': 6,
                    'pyformat': '%Y%m%d',
                    'title': 'I08',
                    'type': 'string'},
            'e16': {'convert': 'Decimal',
                    'data_type_code': 'N0',
                    'maxLength': 6,
                    'minLength': 1,
                    'scale': '?',
                    'title': 'I16',
                    'type': 'string',
                    'x-scale': 0}
        }
    }


def test_loop_annotated():
    class SomeSegment(base.Segment):
        """Some Segment"""
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
    class SomeLoop(base.Loop):
        """Some Loop"""
        some_segment: SomeSegment
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("SS|01|:~", element_sep="|", array_sep=":", segment_sep="~")

    p = base.X12Parser(SomeMsg)

    # l = SomeLoop.parse(src)
    l = p.loop_parse(SomeLoop, src)

    assert l.some_segment.e01 == "01"
    assert list(l.segment_iter()) == [['SS', '01'], ]
    assert repr(l) == "SomeLoop(some_segment=SomeSegment(e01='01'))"
    assert base.asdict(l) ==  {'_kind': 'Loop', '_name': 'SomeLoop', 'some_segment': {'_kind': 'Segment', '_name': 'SomeSegment', 'e01': '01'}}

    assert base.schema(SomeLoop) == {
        'class_name': 'SomeLoop',
        'description': 'Some Loop',
        'properties': {
            'some_segment': {
                'class_name': 'SomeSegment',
                'description': 'Some Segment',
                'properties': {
                    'e01': {
                        'data_type_code': 'ID',
                        'maxLength': 2,
                        'minLength': 2,
                        'title': 'I01',
                        'type': 'string'}
                },
                'type': 'object'}
        },
        'type': 'object'
    }

def test_loop_attr_build_union():
    class OptSegment1(base.Segment):
        """Optional Segment"""
        _segment_name = "OS1"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
    class OptSegment2(base.Segment):
        """Another Optional Segment"""
        _segment_name = "OS2"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
    class SomeLoop(base.Loop):
        """Some Loop"""
        opt_seg_1: OptSegment1 | None
        opt_seg_2: Annotated[OptSegment2, Title("Opt2")] | None
    class SomeMsg(base.Message):
        l01: SomeLoop
    src = base.Source("OS1|01~\nOS2|02~\n", element_sep="|", segment_sep="~", array_sep="^")

    p = base.X12Parser(SomeMsg)
    # SomeLoop.configure([])

    assert OptSegment1._helpers
    assert OptSegment2._helpers
    # loop = SomeLoop.parse(source)
    l = p.loop_parse(SomeLoop, src)
    assert l.opt_seg_1
    assert l.opt_seg_2
    assert list(l.segment_iter()) ==  [['OS1', '01'], ['OS2', '02']]

def test_message_annotated_1():
    class SomeSegment(base.Segment):
        """Some Segment"""
        _segment_name = "SS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
    class SomeLoop(base.Loop):
        """Some Loop"""
        some_segment: SomeSegment
    class AnotherSegment(base.Segment):
        """Another Segment"""
        _segment_name = "AS"
        e01: Annotated[str, Title("I01"), MinLen(2), MaxLen(2), OtherMeta(data_type_code="ID")]
    class AnotherLoop(base.Loop):
        """Another Loop"""
        another_segment: AnotherSegment
    class TheMessage(base.Message):
        """The Message"""
        some_loop: list[SomeLoop]
        another_loop: AnotherLoop | None

    src = base.Source("SS|01~\nAS|02~\n", element_sep="|", array_sep=":", segment_sep="~")
    p = base.X12Parser(TheMessage)
    # m = TheMessage.parse(src)
    m = p.parse(src)
    assert m
    assert m.some_loop[0].some_segment.e01 == "01"
    assert list(m.segment_iter()) == [['SS', '01'], ['AS', '02']]
    assert repr(m) == "TheMessage(some_loop=[SomeLoop(some_segment=SomeSegment(e01='01'))], another_loop=AnotherLoop(another_segment=AnotherSegment(e01='02')))"
    assert base.asdict(m) == {
        '_kind': 'Message',
        '_name': 'TheMessage',
        'another_loop': {'_kind': 'Loop',
                      '_name': 'AnotherLoop',
                      'another_segment': {'_kind': 'Segment',
                                       '_name': 'AnotherSegment',
                                       'e01': '02'}},
        'some_loop': [{'_kind': 'Loop',
                    '_name': 'SomeLoop',
                    'some_segment': {'_kind': 'Segment',
                                     '_name': 'SomeSegment',
                                     'e01': '01'}}]}
    assert m.json() == '{"_kind": "Message", "_name": "TheMessage", "some_loop": [{"_kind": "Loop", "_name": "SomeLoop", "some_segment": {"_kind": "Segment", "_name": "SomeSegment", "e01": "01"}}], "another_loop": {"_kind": "Loop", "_name": "AnotherLoop", "another_segment": {"_kind": "Segment", "_name": "AnotherSegment", "e01": "02"}}}'

    assert base.schema(TheMessage) == {
        'class_name': 'TheMessage',
        'description': "The Message",
        'properties': {
            'another_loop': {
                'any': [
                    {
                        'class_name': 'AnotherLoop',
                         'description': "Another Loop",
                         'properties': {
                             'another_segment': {
                                 'class_name': 'AnotherSegment',
                                 'description': "Another Segment",
                                 'properties': {
                                     'e01': {
                                         'data_type_code': 'ID',
                                        'maxLength': 2,
                                        'minLength': 2,
                                        'title': 'I01',
                                        'type': 'string'}
                                 },
                                 'type': 'object'}
                         },
                         'type': 'object'
                    },
                    {"type": "null"}
                ]
            },
            'some_loop': {
                'items': {
                    'class_name': 'SomeLoop',
                    'description': "Some Loop",
                    'properties': {
                        'some_segment': {
                            'class_name': 'SomeSegment',
                            'description': "Some Segment",
                            'properties': {
                                'e01': {
                                    'data_type_code': 'ID',
                                   'maxLength': 2,
                                   'minLength': 2,
                                   'title': 'I01',
                                   'type': 'string'}},
                            'type': 'object'}},
                    'type': 'object'},
                'type': 'array'}
        },
        'type': 'object'
    }

def test_demo(capsys: CaptureFixture) -> None:
    """An integration test for many features of the base module."""
    base.demo()
    output, _ = capsys.readouterr()
    assert output.splitlines() == [
        "MSG(isa_loop=[ISA_LOOP(isa=ISA_LOOP_ISA(isa01='00', isa16=':'))])",
        "MSG(isa_loop=[ISA_LOOP(isa=ISA_LOOP_ISA(isa01='00', isa16=':'))])",
    ]



"""
277U.4010.X070
Created 2023-05-19 10:17:35.874506
"""
from .base import *
from .common import *


class ISA_LOOP_ISA(Segment):
    """Interchange Control Header"""
    _segment_name = 'ISA'
    
    isa01: Annotated[I01, Title('Authorization Information Qualifier'), Usage('R'), Position(1), Enumerated(*['00', '03'])]
    
    isa02: Annotated[I02, Title('Authorization Information'), Usage('R'), Position(2)]
    
    isa03: Annotated[I03, Title('Security Information Qualifier'), Usage('R'), Position(3), Enumerated(*['00', '01'])]
    
    isa04: Annotated[I04, Title('Security Information'), Usage('R'), Position(4)]
    
    isa05: Annotated[I05, Title('Interchange ID Qualifier'), Usage('R'), Position(5), Enumerated(*['01', '14', '20', '27', '28', '29', '30', '33', 'ZZ'])]
    
    isa06: Annotated[I06, Title('Interchange Sender ID'), Usage('R'), Position(6)]
    
    isa07: Annotated[I05, Title('Interchange ID Qualifier'), Usage('R'), Position(7), Enumerated(*['01', '14', '20', '27', '28', '29', '30', '33', 'ZZ'])]
    
    isa08: Annotated[I07, Title('Interchange Receiver ID'), Usage('R'), Position(8)]
    
    isa09: Annotated[I08, Title('Interchange Date'), Usage('R'), Position(9)]
    
    isa10: Annotated[I09, Title('Interchange Time'), Usage('R'), Position(10)]
    
    isa11: Annotated[I10, Title('Interchange Control Standards Identifier'), Usage('R'), Position(11), Enumerated(*['U'])]
    
    isa12: Annotated[I11, Title('Interchange Control Version Number'), Usage('R'), Position(12), Enumerated(*['00401'])]
    
    isa13: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(13)]
    
    isa14: Annotated[I13, Title('Acknowledgment Requested'), Usage('R'), Position(14), Enumerated(*['0', '1'])]
    
    isa15: Annotated[I14, Title('Usage Indicator'), Usage('R'), Position(15), Enumerated(*['P', 'T'])]
    
    isa16: Annotated[I15, Title('Component Element Separator'), Usage('R'), Position(16)]


class GS_LOOP_GS(Segment):
    """Functional Group Header"""
    _segment_name = 'GS'
    
    gs01: Annotated[D_479, Title('Functional Identifier Code'), Usage('R'), Position(1), Enumerated(*['HN'])]
    
    gs02: Annotated[D_142, Title("Application Sender's Code"), Usage('R'), Position(2)]
    
    gs03: Annotated[D_124, Title("Application Receiver's Code"), Usage('R'), Position(3)]
    
    gs04: Annotated[D_373, Title('Date'), Usage('R'), Position(4)]
    
    gs05: Annotated[D_337, Title('Time'), Usage('R'), Position(5)]
    
    gs06: Annotated[D_28, Title('Group Control Number'), Usage('R'), Position(6)]
    
    gs07: Annotated[D_455, Title('Responsible Agency Code'), Usage('R'), Position(7), Enumerated(*['X'])]
    
    gs08: Annotated[D_480, Title('Version / Release / Industry Identifier Code'), Usage('R'), Position(8), Enumerated(*['004010X070'])]


class ST_LOOP_ST(Segment):
    """Transaction Set Header"""
    _segment_name = 'ST'
    
    st01: Annotated[D_143, Title('Transaction Set Identifier Code'), Usage('R'), Position(1), Enumerated(*['277'])]
    
    st02: Annotated[D_329, Title('Transaction Set Control Number'), Usage('R'), Position(2)]


class HEADER_BHT(Segment):
    """Beginning of Hierarchical Transaction"""
    _segment_name = 'BHT'
    
    bht01: Annotated[D_1005, Title('Hierarchical Structure Code'), Usage('R'), Position(1), Enumerated(*['0010'])]
    
    bht02: Annotated[D_353, Title('Transaction Set Purpose Code'), Usage('R'), Position(2), Enumerated(*['08'])]
    
    bht03: Annotated[D_127, Title('Originator Application Transaction Identifier'), Usage('R'), Position(3)]
    
    bht04: Annotated[D_373, Title('Transaction Set Creation Date'), Usage('R'), Position(4)]
    
    bht05: Annotated[D_337, Title('Time'), Usage('N'), Position(5)]
    
    bht06: Annotated[D_640, Title('Transaction Type Code'), Usage('R'), Position(6), Enumerated(*['NO'])]


class HEADER(Loop):
    
    bht: Annotated[HEADER_BHT, Title('Beginning of Hierarchical Transaction'), Usage('R'), Position(20), Required(True)]


class L2000A_HL(Segment):
    """Information Source Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('N'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['20'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['1'])]


class L2100A_NM1(Segment):
    """Payer Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['PR'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title('Payer Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Name First'), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Name Middle'), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Name Prefix'), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Name Suffix'), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['21', 'AD', 'FI', 'NI', 'PI', 'PP', 'XV'])]
    
    nm109: Annotated[D_67, Title('Payer Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]


class L2100A_PER(Segment):
    """Payer Contact Information"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title('Contact Function Code'), Usage('R'), Position(1), Enumerated(*['IC'])]
    
    per02: Annotated[D_93, Title('Payer Contact Name'), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title('Communication Number Qualifier'), Usage('R'), Position(3), Enumerated(*['ED', 'EM', 'TE'])]
    
    per04: Annotated[D_364, Title('Communication Number'), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title('Communication Number Qualifier'), Usage('S'), Position(5), Enumerated(*['EX'])]
    
    per06: Annotated[D_364, Title('Communication Number'), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title('Communication Number Qualifier'), Usage('S'), Position(7), Enumerated(*['EX', 'FX'])]
    
    per08: Annotated[D_364, Title('Communication Number'), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title('Contact Inquiry Reference'), Usage('N'), Position(9)]


class L2100A(Loop):
    
    nm1: Annotated[L2100A_NM1, Title('Payer Name'), Usage('R'), Position(50), Required(True)]
    
    per: Annotated[L2100A_PER, Title('Payer Contact Information'), Usage('S'), Position(80), Required(True)]


class L2000B_HL(Segment):
    """Information Receiver Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['21'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['1'])]


class L2100B_NM1(Segment):
    """Information Receiver Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['41'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Information Receiver Last or Organization Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Information Receiver First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Information Receiver Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Information Receiver Name Prefix'), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Information Receiver Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['46', 'FI', 'XX'])]
    
    nm109: Annotated[D_67, Title('Information Receiver Identification Number'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]


class L2100B(Loop):
    
    nm1: Annotated[L2100B_NM1, Title('Information Receiver Name'), Usage('R'), Position(50), Required(True)]


class L2000C_HL(Segment):
    """Service Provider Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['19'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['1'])]


class L2100C_NM1(Segment):
    """Provider Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['1P'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Provider Last or Organization Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Provider First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Provider Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Provider Name Prefix'), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Provider Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['FI', 'SV', 'XX'])]
    
    nm109: Annotated[D_67, Title('Provider Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]


class L2100C(Loop):
    
    nm1: Annotated[L2100C_NM1, Title('Provider Name'), Usage('R'), Position(50), Required(True)]


class L2000D_HL(Segment):
    """Subscriber Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['22'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('R'), Position(4), Enumerated(*['0', '1'])]


class L2000D_DMG(Segment):
    """Subscriber Demographic Information"""
    _segment_name = 'DMG'
    
    dmg01: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(1), Enumerated(*['D8'])]
    
    dmg02: Annotated[D_1251, Title('Subscriber Birth Date'), Usage('R'), Position(2)]
    
    dmg03: Annotated[D_1068, Title('Subscriber Gender Code'), Usage('R'), Position(3), Enumerated(*['F', 'M', 'U'])]
    
    dmg04: Annotated[D_1067, Title('Marital Status Code'), Usage('N'), Position(4)]
    
    dmg05: Annotated[D_1109, Title('Race or Ethnicity Code'), Usage('N'), Position(5)]
    
    dmg06: Annotated[D_1066, Title('Citizenship Status Code'), Usage('N'), Position(6)]
    
    dmg07: Annotated[D_26, Title('Country Code'), Usage('N'), Position(7)]
    
    dmg08: Annotated[D_659, Title('Basis of Verification Code'), Usage('N'), Position(8)]
    
    dmg09: Annotated[D_380, Title('Quantity'), Usage('N'), Position(9)]


class L2100D_NM1(Segment):
    """Subscriber Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['IL', 'QC'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title('Subscriber Last Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Subscriber First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Subscriber Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Subscriber Name Prefix'), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Subscriber Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('R'), Position(8), Enumerated(*['24', 'MI', 'ZZ'])]
    
    nm109: Annotated[D_67, Title('Subscriber Identifier'), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]


class L2100D(Loop):
    
    nm1: Annotated[L2100D_NM1, Title('Subscriber Name'), Usage('R'), Position(50), Required(True)]


class L2200D_TRN(Segment):
    """Claim Submitter Trace Number"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['2'])]
    
    trn02: Annotated[D_127, Title('Trace Number'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200D_C043(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc01_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200D_C043(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc10_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200D_C043(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc11_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200D_STC(Segment):
    """Claim Level Status Information"""
    _segment_name = 'STC'
    
    c043: Annotated[L2200D_C043, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Total Claim Charge Amount'), Usage('R'), Position(4)]
    
    stc05: Annotated[D_782, Title('Claim Payment Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Adjudication or Payment Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7), Enumerated(*['ACH', 'BOP', 'CHK', 'FWT', 'NON'])]
    
    stc08: Annotated[D_373, Title('Check Issue or EFT Effective Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check or EFT Trace Number'), Usage('N'), Position(9)]
    
    c043: Annotated[L2200D_C043, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    c043: Annotated[L2200D_C043, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-Form Message Text'), Usage('N'), Position(12)]


class L2200D_C040(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Payer Claim Identification Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['1K'])]
    
    ref02: Annotated[D_127, Title('Payer Claim Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_C040(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Institutional Bill Type Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['BLT'])]
    
    ref02: Annotated[D_127, Title('Bill Type Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_C040(Composite):
    """Reference Identifier"""
    pass


class L2200D_REF(Segment):
    """Medical Record Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['EA'])]
    
    ref02: Annotated[D_127, Title('Medical Record Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200D_DTP(Segment):
    """Claim Service Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['232'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['RD8'])]
    
    dtp03: Annotated[D_1251, Title('Claim Service Date'), Usage('R'), Position(3)]


class L2220D_C003(Composite):
    """Composite Medical Procedure Identifier"""
    
    svc01_01: Annotated[D_235, Title('Product or Service ID Qualifier'), Usage('R'), Position(1), Enumerated(*['AD', 'CI', 'HC', 'ID', 'IV', 'N1', 'N2', 'N3', 'N4', 'ND', 'NH', 'NU', 'RB'])]
    
    svc01_02: Annotated[D_234, Title('Service Identification Code'), Usage('R'), Position(2)]
    
    svc01_03: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(3)]
    
    svc01_04: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(4)]
    
    svc01_05: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(5)]
    
    svc01_06: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(6)]
    
    svc01_07: Annotated[D_352, Title('Description'), Usage('N'), Position(7)]


class L2220D_C003(Composite):
    """Composite Medical Procedure Identifier"""
    pass


class L2220D_SVC(Segment):
    """Service Line Information"""
    _segment_name = 'SVC'
    
    c003: Annotated[L2220D_C003, Title('Composite Medical Procedure Identifier'), Usage('R'), Position(1), Required(True)]
    
    svc02: Annotated[D_782, Title('Line Item Charge Amount'), Usage('R'), Position(2)]
    
    svc03: Annotated[D_782, Title('Line Item Provider Payment Amount'), Usage('R'), Position(3)]
    
    svc04: Annotated[D_234, Title('Revenue Code'), Usage('S'), Position(4)]
    
    svc05: Annotated[D_380, Title('Quantity'), Usage('N'), Position(5)]
    
    svc07: Annotated[D_380, Title('Original Units of Service Count'), Usage('S'), Position(7)]


class L2220D_C043(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc01_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220D_C043(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc10_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220D_C043(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc11_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220D_STC(Segment):
    """Service Line Status Information"""
    _segment_name = 'STC'
    
    c043: Annotated[L2220D_C043, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Line Item Charge Amount'), Usage('S'), Position(4)]
    
    stc05: Annotated[D_782, Title('Line Item Provider Payment Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    c043: Annotated[L2220D_C043, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    c043: Annotated[L2220D_C043, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-Form Message Text'), Usage('N'), Position(12)]


class L2220D_C040(Composite):
    """Reference Identifier"""
    pass


class L2220D_REF(Segment):
    """Service Line Item Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['FJ'])]
    
    ref02: Annotated[D_127, Title('Line Item Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2220D_DTP(Segment):
    """Service Line Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['RD8'])]
    
    dtp03: Annotated[D_1251, Title('Service Line Date'), Usage('R'), Position(3)]


class L2220D(Loop):
    
    svc: Annotated[L2220D_SVC, Title('Service Line Information'), Usage('R'), Position(180), Required(True)]
    
    stc: Annotated[L2220D_STC, Title('Service Line Status Information'), Usage('S'), Position(190), Required(True)]
    
    ref: Annotated[L2220D_REF, Title('Service Line Item Identification'), Usage('S'), Position(200), Required(True)]
    
    dtp: Annotated[L2220D_DTP, Title('Service Line Date'), Usage('S'), Position(210), Required(True)]


class L2200D(Loop):
    
    trn: Annotated[L2200D_TRN, Title('Claim Submitter Trace Number'), Usage('R'), Position(90), Required(True)]
    
    stc: Annotated[L2200D_STC, Title('Claim Level Status Information'), Usage('R'), Position(100), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Payer Claim Identification Number'), Usage('S'), Position(110), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Institutional Bill Type Identification'), Usage('S'), Position(110), Required(True)]
    
    ref: Annotated[L2200D_REF, Title('Medical Record Identification'), Usage('S'), Position(110), Required(True)]
    
    dtp: Annotated[L2200D_DTP, Title('Claim Service Date'), Usage('S'), Position(120), Required(True)]
    ItemL2220D: TypeAlias = Annotated[L2220D, Title('Service Line Information'), Usage('S'), Position(180), Required(True)]
    l2220d: Annotated[list[ItemL2220D], MinItems(1)]


class L2000E_HL(Segment):
    """Dependent Level"""
    _segment_name = 'HL'
    
    hl01: Annotated[D_628, Title('Hierarchical ID Number'), Usage('R'), Position(1)]
    
    hl02: Annotated[D_734, Title('Hierarchical Parent ID Number'), Usage('R'), Position(2)]
    
    hl03: Annotated[D_735, Title('Hierarchical Level Code'), Usage('R'), Position(3), Enumerated(*['23'])]
    
    hl04: Annotated[D_736, Title('Hierarchical Child Code'), Usage('N'), Position(4)]


class L2000E_DMG(Segment):
    """Dependent Demographic Information"""
    _segment_name = 'DMG'
    
    dmg01: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(1), Enumerated(*['D8'])]
    
    dmg02: Annotated[D_1251, Title('Patient Birth Date'), Usage('R'), Position(2)]
    
    dmg03: Annotated[D_1068, Title('Patient Gender Code'), Usage('R'), Position(3), Enumerated(*['F', 'M', 'U'])]
    
    dmg04: Annotated[D_1067, Title('Marital Status Code'), Usage('N'), Position(4)]
    
    dmg05: Annotated[D_1109, Title('Race or Ethnicity Code'), Usage('N'), Position(5)]
    
    dmg06: Annotated[D_1066, Title('Citizenship Status Code'), Usage('N'), Position(6)]
    
    dmg07: Annotated[D_26, Title('Country Code'), Usage('N'), Position(7)]
    
    dmg08: Annotated[D_659, Title('Basis of Verification Code'), Usage('N'), Position(8)]
    
    dmg09: Annotated[D_380, Title('Quantity'), Usage('N'), Position(9)]


class L2100E_NM1(Segment):
    """Dependent Name"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title('Entity Identifier Code'), Usage('R'), Position(1), Enumerated(*['QC'])]
    
    nm102: Annotated[D_1065, Title('Entity Type Qualifier'), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title('Patient Last Name'), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title('Patient First Name'), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title('Patient Middle Name'), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title('Patient Name Prefix'), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title('Patient Name Suffix'), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title('Identification Code Qualifier'), Usage('S'), Position(8), Enumerated(*['MI', 'ZZ'])]
    
    nm109: Annotated[D_67, Title('Patient Primary Identifier'), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title('Entity Relationship Code'), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title('Entity Identifier Code'), Usage('N'), Position(11)]


class L2100E(Loop):
    
    nm1: Annotated[L2100E_NM1, Title('Dependent Name'), Usage('R'), Position(50), Required(True)]


class L2200E_TRN(Segment):
    """Claim Submitter Trace Number"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title('Trace Type Code'), Usage('R'), Position(1), Enumerated(*['2'])]
    
    trn02: Annotated[D_127, Title('Trace Number'), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title('Originating Company Identifier'), Usage('N'), Position(3)]
    
    trn04: Annotated[D_127, Title('Reference Identification'), Usage('N'), Position(4)]


class L2200E_C043(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc01_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200E_C043(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc10_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200E_C043(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc11_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2200E_STC(Segment):
    """Claim Level Status Information"""
    _segment_name = 'STC'
    
    c043: Annotated[L2200E_C043, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Total Claim Charge Amount'), Usage('R'), Position(4)]
    
    stc05: Annotated[D_782, Title('Claim Payment Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Adjudication or Payment Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7), Enumerated(*['ACH', 'BOP', 'CHK', 'FWT', 'NON'])]
    
    stc08: Annotated[D_373, Title('Check Issue or EFT Effective Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check or EFT Trace Number'), Usage('N'), Position(9)]
    
    c043: Annotated[L2200E_C043, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    c043: Annotated[L2200E_C043, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-Form Message Text'), Usage('N'), Position(12)]


class L2200E_C040(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Payer Claim Identification Number"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['1K'])]
    
    ref02: Annotated[D_127, Title('Payer Claim Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_C040(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Institutional Bill Type Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['BLT'])]
    
    ref02: Annotated[D_127, Title('Bill Type Identifier'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_C040(Composite):
    """Reference Identifier"""
    pass


class L2200E_REF(Segment):
    """Medical Record Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['EA'])]
    
    ref02: Annotated[D_127, Title('Medical Record Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2200E_DTP(Segment):
    """Claim Service Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['232'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['RD8'])]
    
    dtp03: Annotated[D_1251, Title('Claim Service Period'), Usage('R'), Position(3)]


class L2220E_C003(Composite):
    """Composite Medical Procedure Identifier"""
    
    svc01_01: Annotated[D_235, Title('Product or Service ID Qualifier'), Usage('R'), Position(1), Enumerated(*['AD', 'CI', 'HC', 'ID', 'IV', 'N1', 'N2', 'N3', 'N4', 'ND', 'NH', 'NU', 'RB'])]
    
    svc01_02: Annotated[D_234, Title('Service Identification Code'), Usage('R'), Position(2)]
    
    svc01_03: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(3)]
    
    svc01_04: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(4)]
    
    svc01_05: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(5)]
    
    svc01_06: Annotated[D_1339, Title('Procedure Modifier'), Usage('S'), Position(6)]
    
    svc01_07: Annotated[D_352, Title('Description'), Usage('N'), Position(7)]


class L2220E_C003(Composite):
    """Composite Medical Procedure Identifier"""
    pass


class L2220E_SVC(Segment):
    """Service Line Information"""
    _segment_name = 'SVC'
    
    c003: Annotated[L2220E_C003, Title('Composite Medical Procedure Identifier'), Usage('R'), Position(1), Required(True)]
    
    svc02: Annotated[D_782, Title('Line Item Charge Amount'), Usage('R'), Position(2)]
    
    svc03: Annotated[D_782, Title('Line Item Provider Payment Amount'), Usage('R'), Position(3)]
    
    svc04: Annotated[D_234, Title('Revenue Code'), Usage('S'), Position(4)]
    
    svc05: Annotated[D_380, Title('Quantity'), Usage('N'), Position(5)]
    
    svc07: Annotated[D_380, Title('Original Units of Service Count'), Usage('S'), Position(7)]


class L2220E_C043(Composite):
    """Health Care Claim Status"""
    
    stc01_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc01_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc01_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220E_C043(Composite):
    """Health Care Claim Status"""
    
    stc10_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc10_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc10_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220E_C043(Composite):
    """Health Care Claim Status"""
    
    stc11_01: Annotated[D_1271, Title('Health Care Claim Status Category Code'), Usage('R'), Position(1), Enumerated(*claim_status_cat)]
    
    stc11_02: Annotated[D_1271, Title('Health Care Claim Status Code'), Usage('R'), Position(2), Enumerated(*claim_status)]
    
    stc11_03: Annotated[D_98, Title('Entity Identifier Code'), Usage('S'), Position(3), Enumerated(*entity_id)]


class L2220E_STC(Segment):
    """Service Line Status Information"""
    _segment_name = 'STC'
    
    c043: Annotated[L2220E_C043, Title('Health Care Claim Status'), Usage('R'), Position(1), Required(True)]
    
    stc02: Annotated[D_373, Title('Status Information Effective Date'), Usage('R'), Position(2)]
    
    stc03: Annotated[D_306, Title('Action Code'), Usage('N'), Position(3)]
    
    stc04: Annotated[D_782, Title('Line Item Charge Amount'), Usage('S'), Position(4)]
    
    stc05: Annotated[D_782, Title('Line Item Provider Payment Amount'), Usage('N'), Position(5)]
    
    stc06: Annotated[D_373, Title('Date'), Usage('N'), Position(6)]
    
    stc07: Annotated[D_591, Title('Payment Method Code'), Usage('N'), Position(7)]
    
    stc08: Annotated[D_373, Title('Date'), Usage('N'), Position(8)]
    
    stc09: Annotated[D_429, Title('Check Number'), Usage('N'), Position(9)]
    
    c043: Annotated[L2220E_C043, Title('Health Care Claim Status'), Usage('S'), Position(10), Required(True)]
    
    c043: Annotated[L2220E_C043, Title('Health Care Claim Status'), Usage('S'), Position(11), Required(True)]
    
    stc12: Annotated[D_933, Title('Free-Form Message Text'), Usage('N'), Position(12)]


class L2220E_C040(Composite):
    """Reference Identifier"""
    pass


class L2220E_REF(Segment):
    """Service Line Item Identification"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title('Reference Identification Qualifier'), Usage('R'), Position(1), Enumerated(*['FJ'])]
    
    ref02: Annotated[D_127, Title('Line Item Control Number'), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title('Description'), Usage('N'), Position(3)]


class L2220E_DTP(Segment):
    """Service Line Date"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title('Date Time Qualifier'), Usage('R'), Position(1), Enumerated(*['472'])]
    
    dtp02: Annotated[D_1250, Title('Date Time Period Format Qualifier'), Usage('R'), Position(2), Enumerated(*['RD8'])]
    
    dtp03: Annotated[D_1251, Title('Service Date'), Usage('R'), Position(3)]


class L2220E(Loop):
    
    svc: Annotated[L2220E_SVC, Title('Service Line Information'), Usage('R'), Position(180), Required(True)]
    
    stc: Annotated[L2220E_STC, Title('Service Line Status Information'), Usage('S'), Position(190), Required(True)]
    
    ref: Annotated[L2220E_REF, Title('Service Line Item Identification'), Usage('S'), Position(200), Required(True)]
    
    dtp: Annotated[L2220E_DTP, Title('Service Line Date'), Usage('S'), Position(210), Required(True)]


class L2200E(Loop):
    
    trn: Annotated[L2200E_TRN, Title('Claim Submitter Trace Number'), Usage('R'), Position(90), Required(True)]
    
    stc: Annotated[L2200E_STC, Title('Claim Level Status Information'), Usage('R'), Position(100), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Payer Claim Identification Number'), Usage('R'), Position(110), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Institutional Bill Type Identification'), Usage('S'), Position(110), Required(True)]
    
    ref: Annotated[L2200E_REF, Title('Medical Record Identification'), Usage('S'), Position(110), Required(True)]
    
    dtp: Annotated[L2200E_DTP, Title('Claim Service Date'), Usage('S'), Position(120), Required(True)]
    ItemL2220E: TypeAlias = Annotated[L2220E, Title('Service Line Information'), Usage('S'), Position(180), Required(True)]
    l2220e: Annotated[list[ItemL2220E], MinItems(1)]


class L2000E(Loop):
    
    hl: Annotated[L2000E_HL, Title('Dependent Level'), Usage('R'), Position(10), Required(True)]
    
    dmg: Annotated[L2000E_DMG, Title('Dependent Demographic Information'), Usage('R'), Position(40), Required(True)]
    ItemL2100E: TypeAlias = Annotated[L2100E, Title('Dependent Name'), Usage('R'), Position(50), Required(True)]
    l2100e: Annotated[list[ItemL2100E], MinItems(1)]
    ItemL2200E: TypeAlias = Annotated[L2200E, Title('Claim Submitter Trace Number'), Usage('R'), Position(90), Required(True)]
    l2200e: Annotated[list[ItemL2200E], MinItems(1)]


class L2000D(Loop):
    
    hl: Annotated[L2000D_HL, Title('Subscriber Level'), Usage('R'), Position(10), Required(True)]
    
    dmg: Annotated[L2000D_DMG, Title('Subscriber Demographic Information'), Usage('S'), Position(40), Required(True)]
    ItemL2100D: TypeAlias = Annotated[L2100D, Title('Subscriber Name'), Usage('R'), Position(50), Required(True)]
    l2100d: Annotated[list[ItemL2100D], MinItems(1)]
    ItemL2200D: TypeAlias = Annotated[L2200D, Title('Claim Submitter Trace Number'), Usage('S'), Position(90), Required(True)]
    l2200d: Annotated[list[ItemL2200D], MinItems(1)]
    ItemL2000E: TypeAlias = Annotated[L2000E, Title('Dependent Level'), Usage('S'), Position(100), Required(True)]
    l2000e: Annotated[list[ItemL2000E], MinItems(1)]


class L2000C(Loop):
    
    hl: Annotated[L2000C_HL, Title('Service Provider Level'), Usage('R'), Position(10), Required(True)]
    ItemL2100C: TypeAlias = Annotated[L2100C, Title('Provider Name'), Usage('R'), Position(50), Required(True)]
    l2100c: Annotated[list[ItemL2100C], MinItems(1)]
    ItemL2000D: TypeAlias = Annotated[L2000D, Title('Subscriber Level'), Usage('R'), Position(60), Required(True)]
    l2000d: Annotated[list[ItemL2000D], MinItems(1)]


class L2000B(Loop):
    
    hl: Annotated[L2000B_HL, Title('Information Receiver Level'), Usage('R'), Position(10), Required(True)]
    ItemL2100B: TypeAlias = Annotated[L2100B, Title('Information Receiver Name'), Usage('R'), Position(50), Required(True)]
    l2100b: Annotated[list[ItemL2100B], MinItems(1)]
    ItemL2000C: TypeAlias = Annotated[L2000C, Title('Service Provider Level'), Usage('R'), Position(60), Required(True)]
    l2000c: Annotated[list[ItemL2000C], MinItems(1)]


class L2000A(Loop):
    
    hl: Annotated[L2000A_HL, Title('Information Source Level'), Usage('R'), Position(10), Required(True)]
    ItemL2100A: TypeAlias = Annotated[L2100A, Title('Payer Name'), Usage('R'), Position(50), Required(True)]
    l2100a: Annotated[list[ItemL2100A], MinItems(1)]
    ItemL2000B: TypeAlias = Annotated[L2000B, Title('Information Receiver Level'), Usage('R'), Position(60), Required(True)]
    l2000b: Annotated[list[ItemL2000B], MinItems(1)]


class DETAIL(Loop):
    ItemL2000A: TypeAlias = Annotated[L2000A, Title('Information Source Level'), Usage('R'), Position(10), Required(True)]
    l2000a: Annotated[list[ItemL2000A], MinItems(1)]


class FOOTER(Loop):
    pass


class ST_LOOP_SE(Segment):
    """Transaction Set Trailer"""
    _segment_name = 'SE'
    
    se01: Annotated[D_96, Title('Transaction Segment Count'), Usage('R'), Position(1)]
    
    se02: Annotated[D_329, Title('Transaction Set Control Number'), Usage('R'), Position(2)]


class ST_LOOP(Loop):
    
    st: Annotated[ST_LOOP_ST, Title('Transaction Set Header'), Usage('R'), Position(10), Required(True)]
    ItemHeader: TypeAlias = Annotated[HEADER, Title('Table 1 - Header'), Usage('R'), Position(15), Required(True)]
    header: Annotated[list[ItemHeader], MinItems(1)]
    ItemDetail: TypeAlias = Annotated[DETAIL, Title('Table 2 - Detail'), Usage('S'), Position(20), Required(True)]
    detail: Annotated[list[ItemDetail], MinItems(1)]
    ItemFooter: TypeAlias = Annotated[FOOTER, Title('Table 3 - Footer'), Usage('N'), Position(30)]
    footer: Annotated[list[ItemFooter], MinItems(1)]
    
    se: Annotated[ST_LOOP_SE, Title('Transaction Set Trailer'), Usage('R'), Position(270), Required(True)]


class GS_LOOP_GE(Segment):
    """Functional Group Trailer"""
    _segment_name = 'GE'
    
    ge01: Annotated[D_97, Title('Number of Transaction Sets Included'), Usage('R'), Position(1)]
    
    ge02: Annotated[D_28, Title('Group Control Number'), Usage('R'), Position(2)]


class GS_LOOP(Loop):
    
    gs: Annotated[GS_LOOP_GS, Title('Functional Group Header'), Usage('R'), Position(10), Required(True)]
    ItemSt_Loop: TypeAlias = Annotated[ST_LOOP, Title('Transaction Set Header'), Usage('R'), Position(20), Required(True)]
    st_loop: Annotated[list[ItemSt_Loop], MinItems(1)]
    
    ge: Annotated[GS_LOOP_GE, Title('Functional Group Trailer'), Usage('R'), Position(30), Required(True)]


class ISA_LOOP_TA1(Segment):
    """Interchange Acknowledgement"""
    _segment_name = 'TA1'
    
    ta101: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(1)]
    
    ta102: Annotated[I08, Title('Interchange Date'), Usage('R'), Position(2)]
    
    ta103: Annotated[I09, Title('Interchange Time'), Usage('R'), Position(3)]
    
    ta104: Annotated[I17, Title('Interchange Acknowledgement Code'), Usage('R'), Position(4), Enumerated(*['A', 'E', 'R'])]
    
    ta105: Annotated[I18, Title('Interchange Note Code'), Usage('R'), Position(5), Enumerated(*['000', '001', '002', '003', '004', '005', '006', '007', '008', '009', '010', '011', '012', '013', '014', '015', '016', '017', '018', '019', '020', '021', '022', '023', '024', '025', '026', '027', '028', '029', '030', '031'])]


class ISA_LOOP_IEA(Segment):
    """Interchange Control Trailer"""
    _segment_name = 'IEA'
    
    iea01: Annotated[I16, Title('Number of Included Functional Groups'), Usage('R'), Position(1)]
    
    iea02: Annotated[I12, Title('Interchange Control Number'), Usage('R'), Position(2)]


class ISA_LOOP(Loop):
    
    isa: Annotated[ISA_LOOP_ISA, Title('Interchange Control Header'), Usage('R'), Position(10), Required(True)]
    ItemGs_Loop: TypeAlias = Annotated[GS_LOOP, Title('Functional Group Header'), Usage('R'), Position(20), Required(True)]
    gs_loop: Annotated[list[ItemGs_Loop], MinItems(1)]
    
    ta1: Annotated[ISA_LOOP_TA1, Title('Interchange Acknowledgement'), Usage('S'), Position(20), Required(True)]
    
    iea: Annotated[ISA_LOOP_IEA, Title('Interchange Control Trailer'), Usage('R'), Position(30), Required(True)]


class MSG277(Message):
    """HIPAA Health Care Claim Unsolicited Status Response X070-277U"""
    ItemIsa_Loop: TypeAlias = Annotated[ISA_LOOP, Title('Interchange Control Header'), Usage('R'), Position(1), Required(True)]
    isa_loop: Annotated[list[ItemIsa_Loop], MinItems(1)]



"""
835.5010.X221.A1.v2
Created 2023-05-19 10:17:36.002838
"""
from .base import *
from .common import *


class ST_LOOP_ST(Segment):
    """"""
    _segment_name = 'ST'
    
    st01: Annotated[D_143, Title(''), Usage('R'), Position(1), Enumerated(*['835'])]
    
    st02: Annotated[D_329, Title(''), Usage('R'), Position(2)]
    
    st03: Annotated[D_1705, Title(''), Usage('N'), Position(3), Enumerated(*['005010X221A1'])]


class HEADER_BPR(Segment):
    """"""
    _segment_name = 'BPR'
    
    bpr01: Annotated[D_305, Title(''), Usage('R'), Position(1), Enumerated(*['C', 'D', 'H', 'I', 'P', 'U', 'X'])]
    
    bpr02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    bpr03: Annotated[D_478, Title(''), Usage('R'), Position(3), Enumerated(*['C', 'D'])]
    
    bpr04: Annotated[D_591, Title(''), Usage('R'), Position(4), Enumerated(*['ACH', 'BOP', 'CHK', 'FWT', 'NON'])]
    
    bpr05: Annotated[D_812, Title(''), Usage('S'), Position(5), Enumerated(*['CCP', 'CTX'])]
    
    bpr06: Annotated[D_506, Title(''), Usage('S'), Position(6), Enumerated(*['01', '04'])]
    
    bpr07: Annotated[D_507, Title(''), Usage('S'), Position(7)]
    
    bpr08: Annotated[D_569, Title(''), Usage('S'), Position(8), Enumerated(*['DA'])]
    
    bpr09: Annotated[D_508, Title(''), Usage('S'), Position(9)]
    
    bpr10: Annotated[D_509, Title(''), Usage('S'), Position(10)]
    
    bpr11: Annotated[D_510, Title(''), Usage('S'), Position(11)]
    
    bpr12: Annotated[D_506, Title(''), Usage('S'), Position(12), Enumerated(*['01', '04'])]
    
    bpr13: Annotated[D_507, Title(''), Usage('S'), Position(13)]
    
    bpr14: Annotated[D_569, Title(''), Usage('S'), Position(14), Enumerated(*['DA', 'SG'])]
    
    bpr15: Annotated[D_508, Title(''), Usage('S'), Position(15)]
    
    bpr16: Annotated[D_373, Title(''), Usage('R'), Position(16)]
    
    bpr17: Annotated[D_1048, Title(''), Usage('N'), Position(17)]
    
    bpr18: Annotated[D_506, Title(''), Usage('N'), Position(18)]
    
    bpr19: Annotated[D_507, Title(''), Usage('N'), Position(19)]
    
    bpr20: Annotated[D_569, Title(''), Usage('N'), Position(20)]
    
    bpr21: Annotated[D_508, Title(''), Usage('N'), Position(21)]


class HEADER_TRN(Segment):
    """"""
    _segment_name = 'TRN'
    
    trn01: Annotated[D_481, Title(''), Usage('R'), Position(1), Enumerated(*['1'])]
    
    trn02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    trn03: Annotated[D_509, Title(''), Usage('R'), Position(3)]
    
    trn04: Annotated[D_127, Title(''), Usage('S'), Position(4)]


class HEADER_CUR(Segment):
    """"""
    _segment_name = 'CUR'
    
    cur01: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['PR'])]
    
    cur02: Annotated[D_100, Title(''), Usage('R'), Position(2), Enumerated(*currency)]
    
    cur03: Annotated[D_280, Title(''), Usage('N'), Position(3)]
    
    cur04: Annotated[D_98, Title(''), Usage('N'), Position(4)]
    
    cur05: Annotated[D_100, Title(''), Usage('N'), Position(5)]
    
    cur06: Annotated[D_669, Title(''), Usage('N'), Position(6)]
    
    cur07: Annotated[D_374, Title(''), Usage('N'), Position(7)]
    
    cur08: Annotated[D_373, Title(''), Usage('N'), Position(8)]
    
    cur09: Annotated[D_337, Title(''), Usage('N'), Position(9)]
    
    cur10: Annotated[D_374, Title(''), Usage('N'), Position(10)]
    
    cur11: Annotated[D_373, Title(''), Usage('N'), Position(11)]
    
    cur12: Annotated[D_337, Title(''), Usage('N'), Position(12)]
    
    cur13: Annotated[D_374, Title(''), Usage('N'), Position(13)]
    
    cur14: Annotated[D_373, Title(''), Usage('N'), Position(14)]
    
    cur15: Annotated[D_337, Title(''), Usage('N'), Position(15)]
    
    cur16: Annotated[D_374, Title(''), Usage('N'), Position(16)]
    
    cur17: Annotated[D_373, Title(''), Usage('N'), Position(17)]
    
    cur18: Annotated[D_337, Title(''), Usage('N'), Position(18)]
    
    cur19: Annotated[D_374, Title(''), Usage('N'), Position(19)]
    
    cur20: Annotated[D_373, Title(''), Usage('N'), Position(20)]
    
    cur21: Annotated[D_337, Title(''), Usage('N'), Position(21)]


class HEADER_REF04(Composite):
    """"""
    pass


class HEADER_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['EV'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class HEADER_REF04(Composite):
    """"""
    pass


class HEADER_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['F2'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class HEADER_DTM(Segment):
    """"""
    _segment_name = 'DTM'
    
    dtm01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['405'])]
    
    dtm02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    dtm03: Annotated[D_337, Title(''), Usage('N'), Position(3)]
    
    dtm04: Annotated[D_623, Title(''), Usage('N'), Position(4)]
    
    dtm05: Annotated[D_1250, Title(''), Usage('N'), Position(5)]
    
    dtm06: Annotated[D_1251, Title(''), Usage('N'), Position(6)]


class L1000A_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['PR'])]
    
    n102: Annotated[D_93, Title(''), Usage('R'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('S'), Position(3), Enumerated(*['XV'])]
    
    n104: Annotated[D_67, Title(''), Usage('S'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L1000A_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L1000A_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L1000A_REF04(Composite):
    """"""
    pass


class L1000A_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['2U', 'EO', 'HI', 'NF'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L1000A_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['CX'])]
    
    per02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('S'), Position(3), Enumerated(*['EM', 'FX', 'TE'])]
    
    per04: Annotated[D_364, Title(''), Usage('S'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['EM', 'EX', 'FX', 'TE'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['EX'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L1000A_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['BL'])]
    
    per02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('S'), Position(3), Enumerated(*['EM', 'TE', 'UR'])]
    
    per04: Annotated[D_364, Title(''), Usage('S'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['EM', 'EX', 'FX', 'TE', 'UR'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['EM', 'EX', 'FX', 'UR'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L1000A_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['IC'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['UR'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('N'), Position(5)]
    
    per06: Annotated[D_364, Title(''), Usage('N'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('N'), Position(7)]
    
    per08: Annotated[D_364, Title(''), Usage('N'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L1000A(Loop):
    
    n1: Annotated[L1000A_N1, Title(''), Usage('R'), Position(800), Syntax(['R0203', 'P0304']), Required(True)]
    
    n3: Annotated[L1000A_N3, Title(''), Usage('R'), Position(1000), Required(True)]
    
    n4: Annotated[L1000A_N4, Title(''), Usage('R'), Position(1100), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]
    ItemRef: TypeAlias = Annotated[L1000A_REF, Title(''), Usage('S'), Position(1200), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(4)]
    
    per: Annotated[L1000A_PER, Title(''), Usage('S'), Position(1300), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    ItemPer: TypeAlias = Annotated[L1000A_PER, Title(''), Usage('R'), Position(1300), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    per: Annotated[list[ItemPer], MinItems(1)]
    
    per: Annotated[L1000A_PER, Title(''), Usage('S'), Position(1300), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]


class L1000B_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['PE'])]
    
    n102: Annotated[D_93, Title(''), Usage('R'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('R'), Position(3), Enumerated(*['FI', 'XV', 'XX'])]
    
    n104: Annotated[D_67, Title(''), Usage('R'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L1000B_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L1000B_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L1000B_REF04(Composite):
    """"""
    pass


class L1000B_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['0B', 'D3', 'PQ', 'TJ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L1000B_RDM04(Composite):
    """"""
    pass


class L1000B_RDM05(Composite):
    """"""
    pass


class L1000B_RDM(Segment):
    """"""
    _segment_name = 'RDM'
    
    rdm01: Annotated[D_756, Title(''), Usage('R'), Position(1), Enumerated(*['BM', 'EM', 'FT', 'OL'])]
    
    rdm02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    rdm03: Annotated[D_364, Title(''), Usage('S'), Position(3)]


class L1000B(Loop):
    
    n1: Annotated[L1000B_N1, Title(''), Usage('R'), Position(800), Syntax(['R0203', 'P0304']), Required(True)]
    
    n3: Annotated[L1000B_N3, Title(''), Usage('S'), Position(1000), Required(True)]
    
    n4: Annotated[L1000B_N4, Title(''), Usage('S'), Position(1100), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]
    ItemRef: TypeAlias = Annotated[L1000B_REF, Title(''), Usage('S'), Position(1200), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MinItems(1)]
    
    rdm: Annotated[L1000B_RDM, Title(''), Usage('S'), Position(1400), Required(True)]


class HEADER(Loop):
    
    bpr: Annotated[HEADER_BPR, Title(''), Usage('R'), Position(200), Syntax(['P0607', 'C0809', 'P1213', 'C1415', 'P1819', 'C2021']), Required(True)]
    
    trn: Annotated[HEADER_TRN, Title(''), Usage('R'), Position(400), Required(True)]
    
    cur: Annotated[HEADER_CUR, Title(''), Usage('S'), Position(500), Syntax(['C0807', 'C0907', 'L101112', 'C1110', 'C1210', 'L131415', 'C1413', 'C1513', 'L161718', 'C1716', 'C1816', 'L192021', 'C2019', 'C2119']), Required(True)]
    
    ref: Annotated[HEADER_REF, Title(''), Usage('S'), Position(600), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[HEADER_REF, Title(''), Usage('S'), Position(600), Syntax(['R0203']), Required(True)]
    
    dtm: Annotated[HEADER_DTM, Title(''), Usage('S'), Position(700), Syntax(['R020305', 'C0403', 'P0506']), Required(True)]
    ItemL1000A: TypeAlias = Annotated[L1000A, Title(''), Usage('R'), Position(800), Required(True)]
    l1000a: Annotated[list[ItemL1000A], MinItems(1)]
    ItemL1000B: TypeAlias = Annotated[L1000B, Title(''), Usage('R'), Position(1400), Required(True)]
    l1000b: Annotated[list[ItemL1000B], MinItems(1)]


class L2000_LX(Segment):
    """"""
    _segment_name = 'LX'
    
    lx01: Annotated[D_554, Title(''), Usage('R'), Position(1)]


class L2000_TS3(Segment):
    """"""
    _segment_name = 'TS3'
    
    ts301: Annotated[D_127, Title(''), Usage('R'), Position(1)]
    
    ts302: Annotated[D_1331, Title(''), Usage('R'), Position(2)]
    
    ts303: Annotated[D_373, Title(''), Usage('R'), Position(3)]
    
    ts304: Annotated[D_380, Title(''), Usage('R'), Position(4)]
    
    ts305: Annotated[D_782, Title(''), Usage('R'), Position(5)]
    
    ts306: Annotated[D_782, Title(''), Usage('N'), Position(6)]
    
    ts307: Annotated[D_782, Title(''), Usage('N'), Position(7)]
    
    ts308: Annotated[D_782, Title(''), Usage('N'), Position(8)]
    
    ts309: Annotated[D_782, Title(''), Usage('N'), Position(9)]
    
    ts310: Annotated[D_782, Title(''), Usage('N'), Position(10)]
    
    ts311: Annotated[D_782, Title(''), Usage('N'), Position(11)]
    
    ts312: Annotated[D_782, Title(''), Usage('N'), Position(12)]
    
    ts313: Annotated[D_782, Title(''), Usage('S'), Position(13)]
    
    ts314: Annotated[D_782, Title(''), Usage('N'), Position(14)]
    
    ts315: Annotated[D_782, Title(''), Usage('S'), Position(15)]
    
    ts316: Annotated[D_782, Title(''), Usage('N'), Position(16)]
    
    ts317: Annotated[D_782, Title(''), Usage('S'), Position(17)]
    
    ts318: Annotated[D_782, Title(''), Usage('S'), Position(18)]
    
    ts319: Annotated[D_782, Title(''), Usage('N'), Position(19)]
    
    ts320: Annotated[D_782, Title(''), Usage('S'), Position(20)]
    
    ts321: Annotated[D_782, Title(''), Usage('S'), Position(21)]
    
    ts322: Annotated[D_782, Title(''), Usage('S'), Position(22)]
    
    ts323: Annotated[D_380, Title(''), Usage('S'), Position(23)]
    
    ts324: Annotated[D_782, Title(''), Usage('S'), Position(24)]


class L2000_TS2(Segment):
    """"""
    _segment_name = 'TS2'
    
    ts201: Annotated[D_782, Title(''), Usage('S'), Position(1)]
    
    ts202: Annotated[D_782, Title(''), Usage('S'), Position(2)]
    
    ts203: Annotated[D_782, Title(''), Usage('S'), Position(3)]
    
    ts204: Annotated[D_782, Title(''), Usage('S'), Position(4)]
    
    ts205: Annotated[D_782, Title(''), Usage('S'), Position(5)]
    
    ts206: Annotated[D_782, Title(''), Usage('S'), Position(6)]
    
    ts207: Annotated[D_380, Title(''), Usage('S'), Position(7)]
    
    ts208: Annotated[D_782, Title(''), Usage('S'), Position(8)]
    
    ts209: Annotated[D_782, Title(''), Usage('S'), Position(9)]
    
    ts210: Annotated[D_380, Title(''), Usage('S'), Position(10)]
    
    ts211: Annotated[D_380, Title(''), Usage('S'), Position(11)]
    
    ts212: Annotated[D_380, Title(''), Usage('S'), Position(12)]
    
    ts213: Annotated[D_380, Title(''), Usage('S'), Position(13)]
    
    ts214: Annotated[D_380, Title(''), Usage('S'), Position(14)]
    
    ts215: Annotated[D_782, Title(''), Usage('S'), Position(15)]
    
    ts216: Annotated[D_380, Title(''), Usage('S'), Position(16)]
    
    ts217: Annotated[D_782, Title(''), Usage('S'), Position(17)]
    
    ts218: Annotated[D_782, Title(''), Usage('S'), Position(18)]
    
    ts219: Annotated[D_782, Title(''), Usage('S'), Position(19)]


class L2100_CLP(Segment):
    """"""
    _segment_name = 'CLP'
    
    clp01: Annotated[D_1028, Title(''), Usage('R'), Position(1)]
    
    clp02: Annotated[D_1029, Title(''), Usage('R'), Position(2), Enumerated(*['1', '19', '2', '20', '21', '22', '23', '25', '3', '4'])]
    
    clp03: Annotated[D_782, Title(''), Usage('R'), Position(3)]
    
    clp04: Annotated[D_782, Title(''), Usage('R'), Position(4)]
    
    clp05: Annotated[D_782, Title(''), Usage('S'), Position(5)]
    
    clp06: Annotated[D_1032, Title(''), Usage('R'), Position(6), Enumerated(*['12', '13', '14', '15', '16', '17', 'AM', 'CH', 'DS', 'HM', 'LM', 'MA', 'MB', 'MC', 'OF', 'TV', 'VA', 'WC', 'ZZ'])]
    
    clp07: Annotated[D_127, Title(''), Usage('R'), Position(7)]
    
    clp08: Annotated[D_1331, Title(''), Usage('S'), Position(8)]
    
    clp09: Annotated[D_1325, Title(''), Usage('S'), Position(9)]
    
    clp10: Annotated[D_1352, Title(''), Usage('N'), Position(10)]
    
    clp11: Annotated[D_1354, Title(''), Usage('S'), Position(11)]
    
    clp12: Annotated[D_380, Title(''), Usage('S'), Position(12)]
    
    clp13: Annotated[D_954, Title(''), Usage('S'), Position(13)]
    
    clp14: Annotated[D_1073, Title(''), Usage('N'), Position(14)]


class L2100_CAS(Segment):
    """"""
    _segment_name = 'CAS'
    
    cas01: Annotated[D_1033, Title(''), Usage('R'), Position(1), Enumerated(*['CO', 'OA', 'PI', 'PR'])]
    
    cas02: Annotated[D_1034, Title(''), Usage('R'), Position(2)]
    
    cas03: Annotated[D_782, Title(''), Usage('R'), Position(3)]
    
    cas04: Annotated[D_380, Title(''), Usage('S'), Position(4)]
    
    cas05: Annotated[D_1034, Title(''), Usage('S'), Position(5)]
    
    cas06: Annotated[D_782, Title(''), Usage('S'), Position(6)]
    
    cas07: Annotated[D_380, Title(''), Usage('S'), Position(7)]
    
    cas08: Annotated[D_1034, Title(''), Usage('S'), Position(8)]
    
    cas09: Annotated[D_782, Title(''), Usage('S'), Position(9)]
    
    cas10: Annotated[D_380, Title(''), Usage('S'), Position(10)]
    
    cas11: Annotated[D_1034, Title(''), Usage('S'), Position(11)]
    
    cas12: Annotated[D_782, Title(''), Usage('S'), Position(12)]
    
    cas13: Annotated[D_380, Title(''), Usage('S'), Position(13)]
    
    cas14: Annotated[D_1034, Title(''), Usage('S'), Position(14)]
    
    cas15: Annotated[D_782, Title(''), Usage('S'), Position(15)]
    
    cas16: Annotated[D_380, Title(''), Usage('S'), Position(16)]
    
    cas17: Annotated[D_1034, Title(''), Usage('S'), Position(17)]
    
    cas18: Annotated[D_782, Title(''), Usage('S'), Position(18)]
    
    cas19: Annotated[D_380, Title(''), Usage('S'), Position(19)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['QC'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'HN', 'II', 'MI', 'MR'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['IL'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('R'), Position(8), Enumerated(*['FI', 'II', 'MI'])]
    
    nm109: Annotated[D_67, Title(''), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['74'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['C'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['82'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('R'), Position(8), Enumerated(*['BD', 'BS', 'FI', 'MC', 'PC', 'SL', 'UP', 'XX'])]
    
    nm109: Annotated[D_67, Title(''), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['TT'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('R'), Position(8), Enumerated(*['AD', 'FI', 'NI', 'PI', 'PP', 'XV'])]
    
    nm109: Annotated[D_67, Title(''), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['PR'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('R'), Position(8), Enumerated(*['AD', 'FI', 'NI', 'PI', 'PP', 'XV'])]
    
    nm109: Annotated[D_67, Title(''), Usage('R'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['GB'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['FI', 'II', 'MI'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100_MIA(Segment):
    """"""
    _segment_name = 'MIA'
    
    mia01: Annotated[D_380, Title(''), Usage('R'), Position(1)]
    
    mia02: Annotated[D_782, Title(''), Usage('S'), Position(2)]
    
    mia03: Annotated[D_380, Title(''), Usage('S'), Position(3)]
    
    mia04: Annotated[D_782, Title(''), Usage('S'), Position(4)]
    
    mia05: Annotated[D_127, Title(''), Usage('S'), Position(5), Enumerated(*remark_code)]
    
    mia06: Annotated[D_782, Title(''), Usage('S'), Position(6)]
    
    mia07: Annotated[D_782, Title(''), Usage('S'), Position(7)]
    
    mia08: Annotated[D_782, Title(''), Usage('S'), Position(8)]
    
    mia09: Annotated[D_782, Title(''), Usage('S'), Position(9)]
    
    mia10: Annotated[D_782, Title(''), Usage('S'), Position(10)]
    
    mia11: Annotated[D_782, Title(''), Usage('S'), Position(11)]
    
    mia12: Annotated[D_782, Title(''), Usage('S'), Position(12)]
    
    mia13: Annotated[D_782, Title(''), Usage('S'), Position(13)]
    
    mia14: Annotated[D_782, Title(''), Usage('S'), Position(14)]
    
    mia15: Annotated[D_380, Title(''), Usage('S'), Position(15)]
    
    mia16: Annotated[D_782, Title(''), Usage('S'), Position(16)]
    
    mia17: Annotated[D_782, Title(''), Usage('S'), Position(17)]
    
    mia18: Annotated[D_782, Title(''), Usage('S'), Position(18)]
    
    mia19: Annotated[D_782, Title(''), Usage('S'), Position(19)]
    
    mia20: Annotated[D_127, Title(''), Usage('S'), Position(20), Enumerated(*remark_code)]
    
    mia21: Annotated[D_127, Title(''), Usage('S'), Position(21), Enumerated(*remark_code)]
    
    mia22: Annotated[D_127, Title(''), Usage('S'), Position(22), Enumerated(*remark_code)]
    
    mia23: Annotated[D_127, Title(''), Usage('S'), Position(23), Enumerated(*remark_code)]
    
    mia24: Annotated[D_782, Title(''), Usage('S'), Position(24)]


class L2100_MOA(Segment):
    """"""
    _segment_name = 'MOA'
    
    moa01: Annotated[D_954, Title(''), Usage('S'), Position(1)]
    
    moa02: Annotated[D_782, Title(''), Usage('S'), Position(2)]
    
    moa03: Annotated[D_127, Title(''), Usage('S'), Position(3), Enumerated(*remark_code)]
    
    moa04: Annotated[D_127, Title(''), Usage('S'), Position(4), Enumerated(*remark_code)]
    
    moa05: Annotated[D_127, Title(''), Usage('S'), Position(5), Enumerated(*remark_code)]
    
    moa06: Annotated[D_127, Title(''), Usage('S'), Position(6), Enumerated(*remark_code)]
    
    moa07: Annotated[D_127, Title(''), Usage('S'), Position(7), Enumerated(*remark_code)]
    
    moa08: Annotated[D_782, Title(''), Usage('S'), Position(8)]
    
    moa09: Annotated[D_782, Title(''), Usage('S'), Position(9)]


class L2100_REF04(Composite):
    """"""
    pass


class L2100_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['1L', '1W', '28', '6P', '9A', '9C', 'BB', 'CE', 'EA', 'F8', 'G1', 'G3', 'IG', 'SY'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2100_REF04(Composite):
    """"""
    pass


class L2100_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['0B', '1A', '1B', '1C', '1D', '1G', '1H', '1J', 'D3', 'G2', 'LU'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2100_DTM(Segment):
    """"""
    _segment_name = 'DTM'
    
    dtm01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['232', '233'])]
    
    dtm02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    dtm03: Annotated[D_337, Title(''), Usage('N'), Position(3)]
    
    dtm04: Annotated[D_623, Title(''), Usage('N'), Position(4)]
    
    dtm05: Annotated[D_1250, Title(''), Usage('N'), Position(5)]
    
    dtm06: Annotated[D_1251, Title(''), Usage('N'), Position(6)]


class L2100_DTM(Segment):
    """"""
    _segment_name = 'DTM'
    
    dtm01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['036'])]
    
    dtm02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    dtm03: Annotated[D_337, Title(''), Usage('N'), Position(3)]
    
    dtm04: Annotated[D_623, Title(''), Usage('N'), Position(4)]
    
    dtm05: Annotated[D_1250, Title(''), Usage('N'), Position(5)]
    
    dtm06: Annotated[D_1251, Title(''), Usage('N'), Position(6)]


class L2100_DTM(Segment):
    """"""
    _segment_name = 'DTM'
    
    dtm01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['050'])]
    
    dtm02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    dtm03: Annotated[D_337, Title(''), Usage('N'), Position(3)]
    
    dtm04: Annotated[D_623, Title(''), Usage('N'), Position(4)]
    
    dtm05: Annotated[D_1250, Title(''), Usage('N'), Position(5)]
    
    dtm06: Annotated[D_1251, Title(''), Usage('N'), Position(6)]


class L2100_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['CX'])]
    
    per02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['EM', 'FX', 'TE'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['EM', 'EX', 'FX', 'TE'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['EX'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100_AMT(Segment):
    """"""
    _segment_name = 'AMT'
    
    amt01: Annotated[D_522, Title(''), Usage('R'), Position(1), Enumerated(*['AU', 'D8', 'DY', 'F5', 'I', 'NL', 'T', 'T2', 'ZK', 'ZL', 'ZM', 'ZN', 'ZO'])]
    
    amt02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    amt03: Annotated[D_478, Title(''), Usage('N'), Position(3)]


class L2100_QTY03(Composite):
    """"""
    pass


class L2100_QTY(Segment):
    """"""
    _segment_name = 'QTY'
    
    qty01: Annotated[D_673, Title(''), Usage('R'), Position(1), Enumerated(*['CA', 'CD', 'LA', 'LE', 'NE', 'NR', 'OU', 'PS', 'VS', 'ZK', 'ZL', 'ZM', 'ZN', 'ZO'])]
    
    qty02: Annotated[D_380, Title(''), Usage('R'), Position(2)]
    
    qty04: Annotated[D_61, Title(''), Usage('N'), Position(4)]


class L2110_SVC01(Composite):
    """"""
    
    svc01_01: Annotated[D_235, Title(''), Usage('R'), Position(1), Enumerated(*['AD', 'ER', 'HC', 'HP', 'IV', 'N4', 'N6', 'NU', 'UI', 'WK'])]
    
    svc01_02: Annotated[D_234, Title(''), Usage('R'), Position(2)]
    
    svc01_03: Annotated[D_1339, Title(''), Usage('S'), Position(3)]
    
    svc01_04: Annotated[D_1339, Title(''), Usage('S'), Position(4)]
    
    svc01_05: Annotated[D_1339, Title(''), Usage('S'), Position(5)]
    
    svc01_06: Annotated[D_1339, Title(''), Usage('S'), Position(6)]
    
    svc01_07: Annotated[D_352, Title(''), Usage('N'), Position(7)]
    
    svc01_08: Annotated[D_234, Title(''), Usage('N'), Position(8)]


class L2110_SVC06(Composite):
    """"""
    
    svc06_01: Annotated[D_235, Title(''), Usage('R'), Position(1), Enumerated(*['AD', 'ER', 'HC', 'HP', 'IV', 'N4', 'NU', 'WK'])]
    
    svc06_02: Annotated[D_234, Title(''), Usage('R'), Position(2)]
    
    svc06_03: Annotated[D_1339, Title(''), Usage('S'), Position(3)]
    
    svc06_04: Annotated[D_1339, Title(''), Usage('S'), Position(4)]
    
    svc06_05: Annotated[D_1339, Title(''), Usage('S'), Position(5)]
    
    svc06_06: Annotated[D_1339, Title(''), Usage('S'), Position(6)]
    
    svc06_07: Annotated[D_352, Title(''), Usage('S'), Position(7)]
    
    svc06_08: Annotated[D_234, Title(''), Usage('N'), Position(8)]


class L2110_SVC(Segment):
    """"""
    _segment_name = 'SVC'
    
    svc01: Annotated[L2110_SVC01, Title(''), Usage('R'), Position(1), Required(True)]
    
    svc02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    svc03: Annotated[D_782, Title(''), Usage('R'), Position(3)]
    
    svc04: Annotated[D_234, Title(''), Usage('S'), Position(4)]
    
    svc05: Annotated[D_380, Title(''), Usage('S'), Position(5)]
    
    svc06: Annotated[L2110_SVC06, Title(''), Usage('S'), Position(6), Required(True)]
    
    svc07: Annotated[D_380, Title(''), Usage('S'), Position(7)]


class L2110_DTM(Segment):
    """"""
    _segment_name = 'DTM'
    
    dtm01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['150', '151', '472'])]
    
    dtm02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    dtm03: Annotated[D_337, Title(''), Usage('N'), Position(3)]
    
    dtm04: Annotated[D_623, Title(''), Usage('N'), Position(4)]
    
    dtm05: Annotated[D_1250, Title(''), Usage('N'), Position(5)]
    
    dtm06: Annotated[D_1251, Title(''), Usage('N'), Position(6)]


class L2110_CAS(Segment):
    """"""
    _segment_name = 'CAS'
    
    cas01: Annotated[D_1033, Title(''), Usage('R'), Position(1), Enumerated(*['CO', 'OA', 'PI', 'PR'])]
    
    cas02: Annotated[D_1034, Title(''), Usage('R'), Position(2)]
    
    cas03: Annotated[D_782, Title(''), Usage('R'), Position(3)]
    
    cas04: Annotated[D_380, Title(''), Usage('S'), Position(4)]
    
    cas05: Annotated[D_1034, Title(''), Usage('S'), Position(5)]
    
    cas06: Annotated[D_782, Title(''), Usage('S'), Position(6)]
    
    cas07: Annotated[D_380, Title(''), Usage('S'), Position(7)]
    
    cas08: Annotated[D_1034, Title(''), Usage('S'), Position(8)]
    
    cas09: Annotated[D_782, Title(''), Usage('S'), Position(9)]
    
    cas10: Annotated[D_380, Title(''), Usage('S'), Position(10)]
    
    cas11: Annotated[D_1034, Title(''), Usage('S'), Position(11)]
    
    cas12: Annotated[D_782, Title(''), Usage('S'), Position(12)]
    
    cas13: Annotated[D_380, Title(''), Usage('S'), Position(13)]
    
    cas14: Annotated[D_1034, Title(''), Usage('S'), Position(14)]
    
    cas15: Annotated[D_782, Title(''), Usage('S'), Position(15)]
    
    cas16: Annotated[D_380, Title(''), Usage('S'), Position(16)]
    
    cas17: Annotated[D_1034, Title(''), Usage('S'), Position(17)]
    
    cas18: Annotated[D_782, Title(''), Usage('S'), Position(18)]
    
    cas19: Annotated[D_380, Title(''), Usage('S'), Position(19)]


class L2110_REF04(Composite):
    """"""
    pass


class L2110_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['1S', 'APC', 'BB', 'E9', 'G1', 'G3', 'LU', 'RB'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2110_REF04(Composite):
    """"""
    pass


class L2110_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['6R'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2110_REF04(Composite):
    """"""
    pass


class L2110_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['0B', '1A', '1B', '1C', '1D', '1G', '1H', '1J', 'D3', 'G2', 'HPI', 'SY', 'TJ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2110_REF04(Composite):
    """"""
    pass


class L2110_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['0K'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2110_AMT(Segment):
    """"""
    _segment_name = 'AMT'
    
    amt01: Annotated[D_522, Title(''), Usage('R'), Position(1), Enumerated(*['B6', 'KH', 'T', 'T2', 'ZK', 'ZL', 'ZM', 'ZN', 'ZO'])]
    
    amt02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    amt03: Annotated[D_478, Title(''), Usage('N'), Position(3)]


class L2110_QTY03(Composite):
    """"""
    pass


class L2110_QTY(Segment):
    """"""
    _segment_name = 'QTY'
    
    qty01: Annotated[D_673, Title(''), Usage('R'), Position(1), Enumerated(*['ZK', 'ZL', 'ZM', 'ZN', 'ZO'])]
    
    qty02: Annotated[D_380, Title(''), Usage('R'), Position(2)]
    
    qty04: Annotated[D_61, Title(''), Usage('N'), Position(4)]


class L2110_LQ(Segment):
    """"""
    _segment_name = 'LQ'
    
    lq01: Annotated[D_1270, Title(''), Usage('R'), Position(1), Enumerated(*['HE', 'RX'])]
    
    lq02: Annotated[D_1271, Title(''), Usage('R'), Position(2)]


class L2110(Loop):
    
    svc: Annotated[L2110_SVC, Title(''), Usage('R'), Position(700), Required(True)]
    ItemDtm: TypeAlias = Annotated[L2110_DTM, Title(''), Usage('S'), Position(800), Syntax(['R020305', 'C0403', 'P0506']), Required(True)]
    dtm: Annotated[list[ItemDtm], MaxItems(2)]
    ItemCas: TypeAlias = Annotated[L2110_CAS, Title(''), Usage('S'), Position(900), Syntax(['L050607', 'C0605', 'C0705', 'L080910', 'C0908', 'C1008', 'L111213', 'C1211', 'C1311', 'L141516', 'C1514', 'C1614', 'L171819', 'C1817', 'C1917']), Required(True)]
    cas: Annotated[list[ItemCas], MaxItems(99)]
    ItemRef: TypeAlias = Annotated[L2110_REF, Title(''), Usage('S'), Position(1000), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(8)]
    
    ref: Annotated[L2110_REF, Title(''), Usage('S'), Position(1000), Syntax(['R0203']), Required(True)]
    ItemRef: TypeAlias = Annotated[L2110_REF, Title(''), Usage('S'), Position(1000), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(10)]
    ItemRef: TypeAlias = Annotated[L2110_REF, Title(''), Usage('S'), Position(1000), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(5)]
    ItemAmt: TypeAlias = Annotated[L2110_AMT, Title(''), Usage('S'), Position(1100), Required(True)]
    amt: Annotated[list[ItemAmt], MaxItems(9)]
    ItemQty: TypeAlias = Annotated[L2110_QTY, Title(''), Usage('S'), Position(1200), Syntax(['E0204', 'R0204']), Required(True)]
    qty: Annotated[list[ItemQty], MaxItems(6)]
    ItemLq: TypeAlias = Annotated[L2110_LQ, Title(''), Usage('S'), Position(1300), Syntax(['C0102']), Required(True)]
    lq: Annotated[list[ItemLq], MaxItems(99)]


class L2100(Loop):
    
    clp: Annotated[L2100_CLP, Title(''), Usage('R'), Position(100), Required(True)]
    ItemCas: TypeAlias = Annotated[L2100_CAS, Title(''), Usage('S'), Position(200), Syntax(['L050607', 'C0605', 'C0705', 'L080910', 'C0908', 'C1008', 'L111213', 'C1211', 'C1311', 'L141516', 'C1514', 'C1614', 'L171819', 'C1817', 'C1917']), Required(True)]
    cas: Annotated[list[ItemCas], MaxItems(99)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    nm1: Annotated[L2100_NM1, Title(''), Usage('S'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    mia: Annotated[L2100_MIA, Title(''), Usage('S'), Position(330), Required(True)]
    
    moa: Annotated[L2100_MOA, Title(''), Usage('S'), Position(350)]
    ItemRef: TypeAlias = Annotated[L2100_REF, Title(''), Usage('S'), Position(400), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(5)]
    ItemRef: TypeAlias = Annotated[L2100_REF, Title(''), Usage('S'), Position(400), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(10)]
    ItemDtm: TypeAlias = Annotated[L2100_DTM, Title(''), Usage('S'), Position(500), Syntax(['R020305', 'C0403', 'P0506']), Required(True)]
    dtm: Annotated[list[ItemDtm], MaxItems(2)]
    
    dtm: Annotated[L2100_DTM, Title(''), Usage('S'), Position(500), Syntax(['R020305', 'C0403', 'P0506']), Required(True)]
    
    dtm: Annotated[L2100_DTM, Title(''), Usage('S'), Position(500), Syntax(['R020305', 'C0403', 'P0506']), Required(True)]
    ItemPer: TypeAlias = Annotated[L2100_PER, Title(''), Usage('S'), Position(600), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    per: Annotated[list[ItemPer], MaxItems(2)]
    ItemAmt: TypeAlias = Annotated[L2100_AMT, Title(''), Usage('S'), Position(620), Required(True)]
    amt: Annotated[list[ItemAmt], MaxItems(13)]
    ItemQty: TypeAlias = Annotated[L2100_QTY, Title(''), Usage('S'), Position(640), Syntax(['E0204', 'R0204']), Required(True)]
    qty: Annotated[list[ItemQty], MaxItems(14)]
    ItemL2110: TypeAlias = Annotated[L2110, Title(''), Usage('S'), Position(700), Required(True)]
    l2110: Annotated[list[ItemL2110], MinItems(999)]


class L2000(Loop):
    
    lx: Annotated[L2000_LX, Title(''), Usage('R'), Position(30), Required(True)]
    
    ts3: Annotated[L2000_TS3, Title(''), Usage('S'), Position(50), Required(True)]
    
    ts2: Annotated[L2000_TS2, Title(''), Usage('S'), Position(70)]
    ItemL2100: TypeAlias = Annotated[L2100, Title(''), Usage('R'), Position(100), Required(True)]
    l2100: Annotated[list[ItemL2100], MinItems(1)]


class DETAIL(Loop):
    ItemL2000: TypeAlias = Annotated[L2000, Title(''), Usage('S'), Position(30), Required(True)]
    l2000: Annotated[list[ItemL2000], MinItems(1)]


class FOOTER_PLB03(Composite):
    """"""
    
    plb03_01: Annotated[D_426, Title(''), Usage('R'), Position(1), Enumerated(*['50', '51', '72', '90', 'AH', 'AM', 'AP', 'B2', 'B3', 'BD', 'BN', 'C5', 'CR', 'CS', 'CT', 'CV', 'CW', 'DM', 'E3', 'FB', 'FC', 'GO', 'HM', 'IP', 'IR', 'IS', 'J1', 'L3', 'L6', 'LE', 'LS', 'OA', 'OB', 'PI', 'PL', 'RA', 'RE', 'SL', 'TL', 'WO', 'WU'])]
    
    plb03_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB05(Composite):
    """"""
    
    plb05_01: Annotated[D_426, Title(''), Usage('R'), Position(1)]
    
    plb05_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB07(Composite):
    """"""
    
    plb07_01: Annotated[D_426, Title(''), Usage('R'), Position(1)]
    
    plb07_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB09(Composite):
    """"""
    
    plb09_01: Annotated[D_426, Title(''), Usage('R'), Position(1)]
    
    plb09_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB11(Composite):
    """"""
    
    plb11_01: Annotated[D_426, Title(''), Usage('R'), Position(1)]
    
    plb11_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB13(Composite):
    """"""
    
    plb13_01: Annotated[D_426, Title(''), Usage('R'), Position(1)]
    
    plb13_02: Annotated[D_127, Title(''), Usage('S'), Position(2)]


class FOOTER_PLB(Segment):
    """"""
    _segment_name = 'PLB'
    
    plb01: Annotated[D_127, Title(''), Usage('R'), Position(1)]
    
    plb02: Annotated[D_373, Title(''), Usage('R'), Position(2)]
    
    plb03: Annotated[FOOTER_PLB03, Title(''), Usage('R'), Position(3), Required(True)]
    
    plb04: Annotated[D_782, Title(''), Usage('R'), Position(4)]
    
    plb05: Annotated[FOOTER_PLB05, Title(''), Usage('S'), Position(5), Required(True)]
    
    plb06: Annotated[D_782, Title(''), Usage('S'), Position(6)]
    
    plb07: Annotated[FOOTER_PLB07, Title(''), Usage('S'), Position(7), Required(True)]
    
    plb08: Annotated[D_782, Title(''), Usage('S'), Position(8)]
    
    plb09: Annotated[FOOTER_PLB09, Title(''), Usage('S'), Position(9), Required(True)]
    
    plb10: Annotated[D_782, Title(''), Usage('S'), Position(10)]
    
    plb11: Annotated[FOOTER_PLB11, Title(''), Usage('S'), Position(11), Required(True)]
    
    plb12: Annotated[D_782, Title(''), Usage('S'), Position(12)]
    
    plb13: Annotated[FOOTER_PLB13, Title(''), Usage('S'), Position(13), Required(True)]
    
    plb14: Annotated[D_782, Title(''), Usage('S'), Position(14)]


class FOOTER(Loop):
    ItemPlb: TypeAlias = Annotated[FOOTER_PLB, Title(''), Usage('S'), Position(100), Syntax(['P0506', 'P0708', 'P0910', 'P1112', 'P1314']), Required(True)]
    plb: Annotated[list[ItemPlb], MinItems(1)]


class ST_LOOP_SE(Segment):
    """"""
    _segment_name = 'SE'
    
    se01: Annotated[D_96, Title(''), Usage('R'), Position(1)]
    
    se02: Annotated[D_329, Title(''), Usage('R'), Position(2)]


class ST_LOOP(Loop):
    
    st: Annotated[ST_LOOP_ST, Title(''), Usage('R'), Position(100), Required(True)]
    ItemHeader: TypeAlias = Annotated[HEADER, Title(''), Usage('R'), Position(110), Required(True)]
    header: Annotated[list[ItemHeader], MinItems(1)]
    ItemDetail: TypeAlias = Annotated[DETAIL, Title(''), Usage('S'), Position(120)]
    detail: Annotated[list[ItemDetail], MinItems(1)]
    ItemFooter: TypeAlias = Annotated[FOOTER, Title(''), Usage('R'), Position(130)]
    footer: Annotated[list[ItemFooter], MinItems(1)]
    
    se: Annotated[ST_LOOP_SE, Title(''), Usage('R'), Position(200), Required(True)]


class MSG835W1(Message):
    """HIPAA Health Care Claim Payment/Advice 005010X221A1 835W1"""
    ItemSt_Loop: TypeAlias = Annotated[ST_LOOP, Title(''), Usage('R'), Position(200), Required(True)]
    st_loop: Annotated[list[ItemSt_Loop], MinItems(1)]



"""Test the x12.annotations"""
from decimal import Decimal
from typing import Annotated, get_type_hints, get_origin, get_args
from x12.annotations import *

def test_format():
    class X:
        some_var: Annotated[str, Format(r"\d+")]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (str, Format(r"\d+"))
    assert repr(th['some_var']) == "typing.Annotated[str, Format('\\\\d+')]"
    assert json_schema(th['some_var']) == {'format': '\\d+', 'type': 'string'}

def test_scale():
    class X:
        some_var: Annotated[Decimal, Scale(2)]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (Decimal, Scale(2))
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[decimal.Decimal, Scale(2)]"
    assert json_schema(th['some_var']) == {'_class': 'Decimal', 'type': 'string', 'x-scale': 2}


def test_minlen_maxlen():
    class X:
        some_var: Annotated[str, MinLen(2), MaxLen(4)]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (str, MinLen(2), MaxLen(4))
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[str, MinLen(2), MaxLen(4)]"
    assert json_schema(th['some_var']) == {'maxLength': 4, 'minLength': 2, 'type': 'string'}


def test_enumerated():
    class X:
        some_var: Annotated[str, Enumerated("This", "That")]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (str, Enumerated("This", "That"))
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[str, Enumerated('This', 'That')]"
    assert json_schema(th['some_var']) == {'enum': ['This', 'That'], 'type': 'string'}


def test_title_usage_position_syntax():
    class X:
        some_var: Annotated[str, Title("Title"), Usage("R"), Position("1"), Syntax(["P2P3"])]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (str, Title("Title"), Usage("R"), Position("1"), Syntax(("P2P3",)))
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[str, Title('Title'), Usage('R'), Position('1'), Syntax(('P2P3',))]"
    assert json_schema(th['some_var']) == {'title': 'Title', 'type': 'string', 'x-position': '1', 'x-syntax': [('P2P3',)], 'x-usage': 'R'}


def test_required_maxItems():
    class X:
        some_var: Annotated[list[str], Required(), MaxItems(2)]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (list[str], Required(), MaxItems(2))
    ann = get_args(th['some_var'])[0]
    assert get_args(ann) == (str,)
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[list[str], Required(), MaxItems(2)]"
    assert json_schema(th['some_var']) == {'maxItems': 2, 'type': 'array', 'items': {'type': 'string'}, 'x-required': True}

def test_title_otherMeta():
    class X:
        some_var: Annotated[str, OtherMeta(name="value")]
    th = get_type_hints(X, include_extras=True)
    assert get_args(th['some_var']) == (str, OtherMeta(name="value"))
    # print(repr(th['some_var']))
    assert repr(th['some_var']) == "typing.Annotated[str, OtherMeta(name='value')]"
    assert json_schema(th['some_var']) == {'name': 'value', 'type': 'string'}
